<?php
/**
 * COD24 API
 * This class is connecting to COD24 API.
 * 
 * Package: COD24 Shipping
 * Author: Rasool Vahdati
 * DateTime: 2022/12/04 00:09:29
 * Last Modified Time: 2023/11/26 15:17:52
 * License: GPL-2.0+
 * License URI: http://www.gnu.org/licenses/gpl-2.0.html
 */ 

if (!defined('ABSPATH')) {
	exit;
} // Exit if accessed directly

if (!class_exists('COD24_API')) {
	class COD24_API
	{

		/**
		 * Construct the plugin object
		 */
		public function __construct()
		{
			add_action( 'admin_bar_menu' , array(  &$this , 'admin_bar_wallet_balance' ), 100 );

		} // END public function __construct()

		/**
		 * Admin bar wallet balance
		 * 
		 * @param admin_bar
		 * @return void
		 */
		public function admin_bar_wallet_balance( $admin_bar )
		{
			global $pagenow;
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Wallet/getWalletAmount';
			$body = [];
			$body = wp_json_encode( $body );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => false,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$result['code'] = wp_remote_retrieve_response_code( $request );
				if( $result['code'] == 401 )
				{
					self::get_token();
				}
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$cod24_wallet_amount = number_format( $result['walletAmount'], 0 ) ." ".get_woocommerce_currency_symbol();
				$admin_bar->add_menu( array( 'id' => 'cod24_wallet_amount' , 'title' => sprintf( __('COD24 Wallet Amount: %s', 'cod24-shipping' ), $cod24_wallet_amount ) ) );   
			}
		}

		/**
		 * Get COD24 token using username and password
		 * @param params
		 * @return token
		 */
		public static function get_token()
		{
			$cod24_username = get_option('wc_cod24_username_panel');
			$cod24_password = get_option('wc_cod24_password_panel');

			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Account/getToken';
			$body = array(
				'userName' => esc_attr( $cod24_username ),
				'password' => esc_attr( $cod24_password ),
			);

			$body = wp_json_encode( $body );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'sslverify'   => false,
				'headers'     => array(
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );
			
			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				error_log( print_r( $request, true ) );
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if(!empty($result['token']))
				{
					update_option('wc_cod24_token', esc_attr( $result['token'] ) );
				}
			}
		}

		/**
		 * Get cod24 rate
		 * 
		 * @param data
		 * @return final_result array | return code & total price of rate
		 */
		public static function get_rate( $data )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Order/getPostPrice';
			$body = array(
				'cityCode' => esc_attr( $data['city'] ),
				'weight' => (int) esc_attr( $data['weight'] ),
				'price' => esc_attr( $data['price'] ),
				'idTypeSend' => (int) esc_attr( $data['send_type'] ),
				'idPayMethod' => (int) esc_attr( $data['pay_type'] ),
				'nonStandardPackage' => false
			);

			$body = wp_json_encode( $body );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['code'] = $result['code'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if( $result['isSuccess'] )
				{
					$final_result['totalPrice'] = $result['totalPostPrice'] + $result['packageingPrice'];
				}
				else
				{
					$final_result['totalPrice'] = 0;
				}
			}

			return $final_result;
		}

		/**
		 * Woocommerce prepare data for send to cod24 panel
		 * 
		 * @param int order_id
		 * @return array data 
		 */
		public static function prepare_data( $order_id )
		{
			$total_weight = 0;
			$service_type = -1;
			$prepare_data = [];
			$current_currency = get_woocommerce_currency();
			$custom_total_weight = get_post_meta( $order_id, '_custom_total_weight', true ); // total weight for calculating of all products
			
			// check weight unit
			$unit = get_option('woocommerce_weight_unit');
            // Check if the weight unit is in grams
            if ($unit != 'g') {
                // The weight is in a different unit, convert to grams.
                $custom_total_weight = $custom_total_weight * 1000; // Convert to grams
            }
			
			$order = wc_get_order( $order_id );
			if (!empty($order->get_shipping_first_name()) && !empty($order->get_shipping_last_name())) {
				$customer_first_name = $order->get_shipping_first_name();
				$customer_last_name = $order->get_shipping_last_name();
				$customer_state = $order->get_shipping_state();
				$customer_city = $order->get_shipping_city();
				$customer_postcode = $order->get_shipping_postcode();
				$customer_address = $order->get_shipping_address_1();
				if (!empty($order->get_shipping_address_2()))
					$customer_address .= " - " . $order->get_shipping_address_2();
			} else {
				$customer_first_name = $order->get_billing_first_name();
				$customer_last_name = $order->get_billing_last_name();
				$customer_state = $order->get_billing_state();
				$customer_city = $order->get_billing_city();
				$customer_postcode = $order->get_billing_postcode();
				$customer_address = $order->get_billing_address_1();
				if (!empty($order->get_billing_address_2()))
					$customer_address .= " - " . $order->get_billing_address_2();
			}
			$customer_phone = str_replace("+98", "0" , $order->get_billing_phone() );
			$customer_email = $order->get_billing_email();

			$user_delivery_address = self::find_city( $customer_city );
		
			if (!empty($order->get_customer_note()))
				$customer_note = $order->get_customer_note();
			else
				$customer_note = '-';
			$products_list = array();
		
			foreach ($order->get_items() as $item) {
				if ($item['product_id'] > 0) {
					
					$name = substr( str_replace(",","**", $item->get_name() ), 0, 250 ); // only get 250 character of name
					$quantity = (int) $item->get_quantity();
					$total_price = $item->get_subtotal();
					$unit_price = (float) ( $total_price / $quantity );
					
					if( $current_currency == 'IRR' ) {
						$final_unit_price = ceil($unit_price * 10);
					} else {
						$final_unit_price = $unit_price;
					}
					
					$_product = $item->get_product();
					$product_obj = array(); // Create a new object using stdClass
					if (!$_product->is_virtual()) {
						$weight = $_product->get_weight();
						if ($unit != 'g') {
                            // The weight is in a different unit, convert to grams.
                            $weight = $weight * 1000; // Convert to grams
                        }
						
						$product_obj['productUserCode'] = (string) $_product->get_id();
						$product_obj['productName'] = $_product->get_name();
						$product_obj['weight'] = (int) $weight;
						$product_obj['count'] = (int) $quantity;
						$product_obj['totalOffAmount'] = 0;
						$product_obj['totalPayAmount'] = (float) $total_price;
						$product_obj['finalPayAmount'] = (float) $total_price;

						$products_list[] = $product_obj; // Add property value to array
					}
				}
			}
		
			// payment type
			$order_payment_method = $order->get_payment_method();
			if ($order_payment_method == 'cod')
				$pay_type = 0;
			else
				$pay_type = 1;
		
			// service type
			$order_shipping_method = $order->get_shipping_method();
			$shipping_items = $order->get_items('shipping');
			if(!empty($shipping_items))
			{
				$shipping_item  = reset($shipping_items);
				$shipping_method_id  = $shipping_item->get_method_id();
				if ($shipping_method_id == 'cod24_sefareshi')
					$service_type = 0;
				elseif ($shipping_method_id == 'cod24_pishtaz')
					$service_type = 1;
				elseif ($shipping_method_id == 'free_shipping')
					$service_type = 1;
				else // for other shipping methods
					$service_type = 1;
			}
			else
			{
				$service_type = 1;
			}
			
			// check shipping cost on delivery
			if(empty($pay_type))
				$pay_type = 1;

			$shipping_cost_on_delivery = get_post_meta( $order_id, '_is_shipping_cost_on_delivery', true );
			if($shipping_cost_on_delivery=='yes')
				$pay_type = 3;

			if($shipping_method_id=='free_shipping')
				$pay_type = 1;

			// change pay type when free shipping and cod
			if($order_payment_method == 'cod')
				$pay_type = 0;
			
			// change pay type when free shipping and cod
			if($shipping_method_id=='free_shipping' && $order_payment_method == 'cod')
				$pay_type = 88;
			
			$prepare_data = array(
				'idOrderShop' => (int) $order->get_id(),
				'cityCode' => $user_delivery_address['city_id'],
				'idTypeSend' => $service_type,
				'idPayMethod' => $pay_type,
				'description' => $customer_note,
				'firstName' => $customer_first_name,
				'lastName' => $customer_last_name,
				'mobile' => $customer_phone,
				'postalCode' => $customer_postcode,
				'nationalCode' => "",
				'address' => $customer_address,
				'nonStandardPackage' => false,
				'finalPayAmountCustomer' => (float) $order->get_subtotal(),
				'requestOrderProducts' => $products_list,
				'totalWeight' => (int) $custom_total_weight
			);

			return $prepare_data;
		}

		/**
		 * Add suspend order on cod24 panel
		 * 
		 * @param order_id int
		 * @return final_result array | return code & total price of rate
		 */
		public static function add_order( $order_id )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Order/addOrder';
			$sanitized_data = self::prepare_data( $order_id );
			
			$body = wp_json_encode( $sanitized_data );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
				if( !empty( $result['errors'] ) )
				{
					$final_result['errors'] = $result['errors'];
				}
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if( $result['isSuccess'] )
				{
					if( !empty( $result['isSuccess'] ) && $result['isSuccess'] )
					{
						$final_result['order_id'] = $result['idOrderShop'];
						$final_result['serial'] = $result['serial'];
					}
					else
					{
						$final_result['code'] = $result['code'];
						$final_result['message'] = $result['message'];
					}
				}
				else
				{
					$final_result['code'] = $result['code'];
					$final_result['message'] = $result['message'];
				}
			}

			return $final_result;
		}

		/**
		 * Add suspend order on cod24 panel
		 * 
		 * @param order_ids array
		 * @return final_result array | return code & total price of rate
		 */
		public static function add_order_batch( $order_ids )
		{
			$final_result_list = [];
			$sanitized_data = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Order/addOrderBatch';
			if( !empty( $order_ids ) )
			{
				foreach($order_ids as $order_id)
				{
					$sanitized_data[] = self::prepare_data( $order_id );
				}
			}
			$body = wp_json_encode( $sanitized_data );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if( !empty( $result ) )
				{
					foreach( $result as $res_item )
					{
						if( !empty( $res_item['isSuccess'] ) && $res_item['isSuccess'] )
						{
							$final_result_list[] = [
								'order_id' => $res_item['idOrderShop'],
								'serial' => $res_item['serial']
							];
						}
						else
						{
							$final_result_list[] = [
								'code' => $res_item['code'],
								'message' => $res_item['message']
							];
						}
					}
				}
			}

			return $final_result_list;
		}

		/**
		 * Suspend order
		 * 
		 * @param serials
		 * @return final_result array | return code & total price of rate
		 */
		public static function suspend_order( $serials )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Order/suspendOrder';

			$serials_list = [];
			if(!empty($serials))
			{
				foreach($serials as $serial)
				{
					$serials_list[] = array( 'serial' => intval( $serial ) );
				}
			}

			$body = wp_json_encode( $serials_list );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if(!empty($result))
				{
					$final_result = $result;
				}
			}

			return $final_result;
		}

		/**
		 * Ready to send order
		 * 
		 * @param serials
		 * @return final_result array | return code & total price of rate
		 */
		public static function ready_to_send_order( $serials )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Order/readyToSend';
			
			$serials_list = [];
			if(!empty($serials))
			{
				foreach($serials as $serial)
				{
					$serials_list[] = array( 'serial' => $serial );
				}
			}

			$body = wp_json_encode( $serials_list );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if(!empty($result))
				{
					$final_result = $result;
				}
			}

			return $final_result;
		}

		/**
		 * Cancel order
		 * 
		 * @param serials
		 * @return final_result array | return code & total price of rate
		 */
		public static function cancel_order( $serials )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Order/cancelOrder';
			
			$serials_list = [];
			if(!empty($serials))
			{
				foreach($serials as $serial)
				{
					$serials_list[] = array( 'serial' => $serial );
				}
			}

			$body = wp_json_encode( $serials_list );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if(!empty($result))
				{
					$final_result = $result;
				}
			}

			return $final_result;
		}

		/**
		 * Factor Preview
		 * 
		 * @param serials
		 * @return final_result factor url preview
		 */
		public static function factor_preview( $serials )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Print/factorPreview';
			$body = wp_json_encode( $serials );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if( !empty( $result['isSuccess'] ) && $result['isSuccess'] && $result['code'] == 0 )
				{
					if( !empty( $result['linkAddress'] ) && !empty( $result['token'] ) )
					{
						// api arguments
						$args = array(
							'method'      => 'POST',
							'timeout'     => 500,
							'sslverify'   => true,
							'headers'     => array(
								'Authorization' => esc_attr( $result['token'] ),
								'Content-Type'  => 'application/json',
							)
						);

						$factor_preview_request = wp_remote_post( esc_url( $result['linkAddress'] ), $args );
						$factor_url = wp_remote_retrieve_body( $factor_preview_request );
						$final_result['url'] = $factor_url;
					}
					else
					{
						$final_result['code'] = $result['code'];
						$final_result['message'] = $result['message'];
					}
				}
			}

			return $final_result;
		}

		/**
		 * Barcode Preview
		 * 
		 * @param serials
		 * @return final_result barcode url preview
		 */
		public static function barcode_preview( $serials )
		{
			$final_result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/Print/barcodePreview';
			$body = wp_json_encode( $serials );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => true,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				$final_result['code'] = wp_remote_retrieve_response_code( $request );
				$final_result['message'] = $result['message'];
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
				if( !empty( $result['isSuccess'] ) && $result['isSuccess'] && $result['code'] == 0 )
				{
					if( !empty( $result['linkAddress'] ) && !empty( $result['token'] ) )
					{
						// api arguments
						$args = array(
							'method'      => 'POST',
							'timeout'     => 500,
							'sslverify'   => true,
							'headers'     => array(
								'Authorization' => esc_attr( $result['token'] ),
								'Content-Type'  => 'application/json',
							)
						);

						$factor_preview_request = wp_remote_post( esc_url( $result['linkAddress'] ), $args );
						$factor_url = wp_remote_retrieve_body( $factor_preview_request );
						$final_result['url'] = $factor_url;
					}
					else
					{
						$final_result['code'] = $result['code'];
						$final_result['message'] = $result['message'];
					}
				}
			}

			return $final_result;
		}

		/**
		 * Get cod24 all states
		 * 
		 * @param void
		 * @return result array | list of all states
		 */
		public static function get_states()
		{
			$result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/State/getStates';
			$body = [];
			$body = wp_json_encode( $body );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => false,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$result['code'] = wp_remote_retrieve_response_code( $request );
				if( $result['code'] == 401 )
				{
					self::get_token();
					self::get_states();
				}
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
			}

			return $result;
		}

		/**
		 * Get all cities
		 * 
		 * @param void
		 * @return result array | list of all cities
		 */
		public static function get_all_cities()
		{
			$result = [];
			$cod24_token = get_option('wc_cod24_token');
			
			// send to api
			$api_protocol = ( get_option('wc_cod24_api_secure_connection_enabled')=='yes' ) ? 'https://' : 'http://';
			$url  = $api_protocol.'api.cod24.ir/api/City/getAllCities';
			$body = [];
			$body = wp_json_encode( $body );

			// api arguments
			$args = array(
				'method'      => 'POST',
				'timeout'     => 500,
				'sslverify'   => false,
				'headers'     => array(
					'Authorization' => 'Bearer '.$cod24_token,
					'Content-Type'  => 'application/json',
				),
				'body'        => $body,
			);

			$request = wp_remote_post( $url, $args );

			if ( is_wp_error( $request ) || wp_remote_retrieve_response_code( $request ) != 200 )
			{
				$result['code'] = wp_remote_retrieve_response_code( $request );
				if( $result['code'] == 401 )
				{
					self::get_token();
					self::get_all_cities();
				}
			}
			else
			{
				$response = wp_remote_retrieve_body( $request );
				$result = json_decode( $response, true );
			}

			return $result;
		}

		/**
		 * Find state
		 * 
		 * @param user_state_code
		 * @return state_code
		 */
		public static function find_state( $user_state_code )
		{
			$data = self::get_states();

			if (in_array($user_state_code, array_column($data, 'statePostCode'))) {
				$user_state_key = array_search( $user_state_code, array_column($data, 'statePostCode'));
				return $data[$user_state_key]['statePostCode'];
			}
		
			return false;
		}

		/**
		 * Get all cities of a state by user state code
		 * 
		 * @param user_state_code
		 * @return founded_cities array | all cities of a state
		 */
		public static function get_cities( $user_state_code )
		{
			$data = self::get_states();

			if (in_array($user_state_code, array_column($data, 'shortNameEn'))) {
				$user_state_key = array_search( $user_state_code, array_column($data, 'shortNameEn'));
				$founded_state_id = $data[$user_state_key]['statePostCode'];

				// Get founded state cities
				$cities_data = self::get_all_cities();
				
				$founded_cities = [];
				foreach ($cities_data as $key => $value) {
					if($value['statePostCode'] == $founded_state_id)
					{
						$founded_cities[] = [
							'state_id' => $value['statePostCode'],
							'city_id' => $value['cityPostCode'],
							'city_name' => $value['cityNameFa']
						];
					}
				}

				return $founded_cities;
			}

			return false;
		}

		/**
		 * Find city by city name
		 * 
		 * @param user_city_name name of city
		 * @return result state_id & city_id
		 */
		public static function find_city( $user_city_name )
		{
			$data = self::get_all_cities();

			if (!empty($user_city_name) && in_array($user_city_name, array_column($data, 'cityNameFa'))) {
				$selected_key = array_search( $user_city_name, array_column($data, 'cityNameFa'));
				return [
					'state_id' => $data[$selected_key]['statePostCode'],
					'city_id'  => $data[$selected_key]['cityPostCode']
				];
			}

			return false;
		}

	} // END class COD24_Shipping
} // END if(!class_exists('COD24_Shipping'))

if (class_exists('COD24_API')) {
	// instantiate the plugin class
	new COD24_API();
}